var ZeedhiInterations = angular.module("ZeedhiInterations", []);

ZeedhiInterations.directive('groupHeader', ["templateManager", function(templateManager) {
	var openGroups = [];
	var lastOpened;
	var openOpen = function() {

		openGroups.forEach(function(group) {
			var groupElement = $("[data-group-id=" + group + "]");
			openGroup(groupElement, group);
		});

		if (openGroups.length) {
			var element = $("[data-group-id=" + openGroups[0] + "]");
			var scrollElement = element.closest("[zh-scroller]");
			var scroller = scrollElement.attr("index");
			if (scroller !== void 0) {
				templateManager.scrollers[scroller].scrollToElement($("[data-group-id=" + lastOpened + "]")[0]);
			} else {
				var top = $("[data-group-id=" + lastOpened + "]").position().top;
				$(scrollElement).scrollTop(top);
			}
		}

	};

	function openGroup(element, id) {

		element.parent().toggleClass('opened');

		var productsContainer = element.next();
		var productsUl = productsContainer.children().eq(0);

		if (element.parent().hasClass('opened')) {

			if (openGroups.indexOf(id) === -1) {
				openGroups.push(id);
			}

			productsContainer[0].style.height = productsUl[0].clientHeight + 'px';
		} else {

			var index = openGroups.indexOf(id);

			if (index !== -1) {
				openGroups.splice(index, 1);
			}

			productsContainer[0].style.height = 0;
		}
	}

	return {
		restrict: 'C',
		link: function($scope, element, attrs, controller) {
			element.on('click', function() {
				openGroup(element, attrs.groupId);
			});

			Util.doAsync(function() {
				element.parent().children().find(".products li").on("click", function() {
					lastOpened = attrs.groupId;
				});
			});

			if ($scope.$last) {
				Util.doAsync(openOpen);
			}

			if (!$scope.widget.clearOpened) {
				$scope.widget.clearOpened = function() {
					lastOpened = undefined;
					openGroups = [];
				};
			}

			var _widget = $scope.widget;
			$scope.$on('$destroy', function() {
				delete _widget.clearOpened;
			});
		}
	};
}]);

ZeedhiInterations.directive('zhPopup', function() {
	return function(scope, elem, attrs) {

		var widget = scope.widget;

		widget.showPopup = function() {
			elem.find(".md-modal").addClass("md-show");
			$('html').addClass("md-perspective");
		};

		widget.hidePopup = function() {
			elem.find(".md-modal").removeClass("md-show");
			$('html').removeClass("md-perspective");
		};

		$scope.$on('$destroy', function() {
			delete widget.showPopup;
			delete widget.hidePopup;
		});
	};
});

ZeedhiInterations.directive('zhToggleClass', function() {
	return function(scope, elem, attrs) {
		var selection = {};
		selection.siblings = elem.siblings;
		selection.closest = elem.closest;
		selection.parents = elem.parents;
		var eventType = attrs.event ? attrs.event : 'click';
		elem.bind(eventType, function(event) {
			attrs.zhToggleClassSelector = attrs.zhToggleClassSelector ? attrs.zhToggleClassSelector : 'siblings';
			var zhToggleClass = attrs.zhToggleClassTo ? selection[attrs.zhToggleClassSelector].call(elem, attrs.zhToggleClassTo) : elem;
			var container = attrs.zhToggleClass;
			var parent;
			if (zhToggleClass.hasClass('disabled') || attrs.categoriaOn === false) {
				event.stopPropagation();
				return;
			}
			if (zhToggleClass.hasClass('selected')) {
				zhToggleClass.removeClass("selected");
				return;
			}
			if (attrs.zhMultiple === undefined || attrs.zhMultiple === false) {
				if (container) {
					parent = zhToggleClass.closest(container);
				} else {
					parent = zhToggleClass.parent();
				}
				var selector = attrs.zhToggleClassTo ? attrs.zhToggleClassTo : '[zh-toggle-class="' + attrs.zhToggleClass + '"]';
				parent.find(selector).removeClass('selected');
			}
			zhToggleClass.addClass('selected');
		});
	};
});

ZeedhiInterations.directive("zhGridRow", function(templateManager) {
	return {
		priority: 500,
		link: function($scope, iElm, iAttrs, controller) {

			iElm.on("click", function(e) {
				if($scope.widget && $scope.rowGroup){
					$scope.widget.rowGroup = $scope.rowGroup;
				}
				$scope.widget.setCurrentRow($scope.row);
				templateManager.menuVisible(false);
				templateManager.updateTemplate();
			});
		}
	};
});

ZeedhiInterations.directive("zhGanttContainer", function($timeout, $rootScope, $compile) {
	return {
		restrict: "C",
		link: function($scope, element, iAttrs, controller) {
			var innerFixedContainer = element.find('.table-container-fixed .zh-inner');
			innerFixedContainer.nonbounce();
			var innerContainer = element.find('.table-container .zh-inner');
			innerContainer.nonbounce();

			var scrollInnerFixedContainer = function() {
				innerContainer.off('scroll');
				innerContainer.scrollTop(innerFixedContainer.scrollTop());
				$timeout(function() {
					innerContainer.on('scroll', scrollInnerContainer);
				}, 0, false);
			};
			innerFixedContainer.on('scroll', scrollInnerFixedContainer);

			var scrollInnerContainer = function() {
				innerFixedContainer.off('scroll');
				innerFixedContainer.scrollTop(innerContainer.scrollTop());
				innerContainer.find("th div.zh-theader").css("margin-left", -9 - innerContainer.scrollLeft() + "px");
				$timeout(function() {
					innerFixedContainer.on('scroll', scrollInnerFixedContainer);
				}, 0, false);
			};
			innerContainer.on('scroll', scrollInnerContainer);

			function getDateInterval(widget) {
				var minDate, maxDate;
				var dateAux;
				var data = widget.dataSource.data;
				for (var i = 0; i < data.length; i++) {
					dateAux = new Date(data[i][widget.fieldPlanStart]);
					if (!minDate || dateAux < minDate) {
						minDate = dateAux;
					}
					if (data[i][widget.fieldRealStart]) {
						dateAux = new Date(data[i][widget.fieldRealStart]);
						if (!minDate || dateAux < minDate) {
							minDate = dateAux;
						}
					}
					dateAux = new Date(data[i][widget.fieldPlanEnd]);
					if (!maxDate || dateAux > maxDate) {
						maxDate = dateAux;
					}
					if (data[i][widget.fieldRealEnd]) {
						dateAux = new Date(data[i][widget.fieldRealEnd]);
						if (!maxDate || dateAux > maxDate) {
							maxDate = dateAux;
						}
					}
				}

				return {
					"min": minDate,
					"max": maxDate
				};
			}

			function getGanttSteps() {
				var steps = [];
				var parentSteps = [];
				var getDateLevel, getInitialDate, getEndDate, getNextStep, getStepLabel, getParentStep;

				var dateInterval = getDateInterval($scope.widget);

				var months = ['Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez'];

				function getDateOfISOWeek(y, w) {
					var simple = new Date(y, 0, 1 + (w - 1) * 7, 1, 0, 0);
					var dow = simple.getDay();
					var ISOweekStart = simple;
					if (dow <= 4)
						ISOweekStart.setDate(simple.getDate() - simple.getDay() + 1);
					else
						ISOweekStart.setDate(simple.getDate() + 8 - simple.getDay());
					return ISOweekStart;
				}

				function dateAdd(date, y, m, d) {
					return new Date(date.getFullYear() + y, date.getMonth() + m, date.getDate() + d, 12, 0, 0);
				}

				switch ($scope.zoomLevel) {
					case "YEAR":
						getDateLevel = function(d) {
							return d.getFullYear();
						};
						getInitialDate = function(y) {
							return new Date(y, 0, 1, 1, 0, 0);
						};
						getEndDate = function(y) {
							return new Date(y, 11, 31, 1, 0, 0);
						};
						getNextStep = function(y) {
							return y + 1;
						};
						getStepLabel = function(y) {
							return y.toString();
						};
						getParentStep = function(y) {
							return "ano";
						};
						break;
					case "QUARTER":
						getDateLevel = function(d) {
							return d.getFullYear() + 'T' + (Math.trunc(d.getMonth() / 3));
						};
						getInitialDate = function(q) {
							return new Date(parseInt(q.substr(0, 4)), parseInt(q.substr(5, 1)) * 3, 1, 1, 0, 0);
						};
						getEndDate = function(q) {
							return new Date(parseInt(q.substr(0, 4)), parseInt(q.substr(5, 1)) * 3 + 3, 0, 1, 0, 0);
						};
						getNextStep = function(q) {
							return getDateLevel(dateAdd(getInitialDate(q), 0, 3, 0));
						};
						getStepLabel = function(q) {
							return 'T' + (parseInt(q.substr(5, 1)) + 1);
						};
						getParentStep = function(q) {
							return q.substr(0, 4);
						};
						break;
					case "MONTH":
						getDateLevel = function(d) {
							return d.getFullYear() + 'M' + ('0' + d.getMonth()).substr(-2);
						};
						getInitialDate = function(m) {
							return new Date(parseInt(m.substr(0, 4)), parseInt(m.substr(5, 2)), 1, 1, 0, 0);
						};
						getEndDate = function(m) {
							return new Date(parseInt(m.substr(0, 4)), parseInt(m.substr(5, 2)) + 1, 0, 1, 0, 0);
						};
						getNextStep = function(m) {
							return getDateLevel(dateAdd(getInitialDate(m), 0, 1, 0));
						};
						getStepLabel = function(m) {
							return months[parseInt(m.substr(5, 2))];
						};
						getParentStep = function(m) {
							return m.substr(0, 4);
						};
						break;
					case "WEEK":
						getDateLevel = function(d) {
							var janOne = new Date(d.getFullYear(), 0, 1, 1, 0, 0);
							var weekNumber = Math.ceil((((d - janOne) / 86400000) + janOne.getDay()) / 7);
							return d.getFullYear() + 'S' + ('0' + weekNumber).substr(-2);
						};
						getInitialDate = function(w) {
							return getDateOfISOWeek(parseInt(w.substr(0, 4)), parseInt(w.substr(5, 2)));
						};
						getEndDate = function(w) {
							var nextWeek = getDateOfISOWeek(parseInt(w.substr(0, 4)), parseInt(w.substr(5, 2)) + 1);
							nextWeek.setDate(nextWeek.getDate() - 1);
							return nextWeek;
						};
						getNextStep = function(w) {
							return getDateLevel(dateAdd(getInitialDate(w), 0, 0, 7));
						};
						getStepLabel = function(w) {
							return w.substr(4, 3);
						};
						getParentStep = function(w) {
							return months[getDateOfISOWeek(parseInt(w.substr(0, 4)), parseInt(w.substr(5, 2))).getMonth()] + ' ' + parseInt(w.substr(0, 4));
						};
						break;
					case "DAY":
						getDateLevel = function(d) {
							return d.getFullYear() + 'M' + ('0' + d.getMonth()).substr(-2) + 'D' + ('0' + d.getDate()).substr(-2);
						};
						getInitialDate = function(d) {
							return new Date(parseInt(d.substr(0, 4)), parseInt(d.substr(5, 2)), parseInt(d.substr(8, 2)), 1, 0, 0);
						};
						getEndDate = function(d) {
							return new Date(parseInt(d.substr(0, 4)), parseInt(d.substr(5, 2)), parseInt(d.substr(8, 2)), 1, 0, 0);
						};
						getNextStep = function(d) {
							return getDateLevel(dateAdd(getInitialDate(d), 0, 0, 1));
						};
						getStepLabel = function(d) {
							return d.substr(8, 2);
						};
						getParentStep = function(d) {
							return months[getInitialDate(d).getMonth()] + ' ' + parseInt(d.substr(0, 4));
						};
						break;
				}

				var today = new Date().setHours(0, 0, 0, 0);
				var lastParent = null;
				lastInterval = null;
				for (var i = getDateLevel(dateInterval.min); i <= getDateLevel(dateInterval.max); i = getNextStep(i)) {
					steps.push({
						"label": getStepLabel(i),
						"begin": getInitialDate(i).setHours(0),
						"end": getEndDate(i).setHours(0),
						"firstChild": lastParent !== getParentStep(i),
						"today": lastInterval !== null && today >= lastInterval.begin && today <= lastInterval.end,
						"weekday": $scope.zoomLevel == "DAY" ? getInitialDate(i).getDay() : 'X'
					});
					lastInterval = {
						begin: getInitialDate(i).setHours(0),
						end: getEndDate(i).setHours(0)
					};

					if (lastParent !== getParentStep(i)) {
						lastParent = getParentStep(i);
						parentSteps.push({
							"label": lastParent,
							"colSpan": 1
						});
					} else {
						parentSteps[parentSteps.length - 1].colSpan = parentSteps[parentSteps.length - 1].colSpan + 1;
					}
				}

				return {
					"steps": steps,
					"parentSteps": parentSteps
				};
			}

			function cleanGantt() {
				element.find('.table-container table tbody').empty();
			}

			var building = false;

			function buildGantt(rowIdx) {
				var html = '',
					planned, real, late;
				var data = $scope.widget.plainTreeData;
				var steps = $scope.ganttSteps.steps;
				var inRange;

				if (rowIdx === 0) {
					cleanGantt();
				}

				if (rowIdx < data.length) {
					late = isLate(data[rowIdx]);

					html = '';
					planned = '';
					real = '';
					for (var j = 0; j < steps.length; j++) {
						inRange = isInRange(data[rowIdx], steps[j], 'planned');
						planned += '<td class="' + (inRange ? 'planned ' : '') + (steps[j].today ? 'today ' : '') + ('day' + steps[j].weekday) + '">';
						planned += '<span>&nbsp;</span>';
						planned += '</td>';

						inRange = isInRange(data[rowIdx], steps[j], 'real');
						real += '<td class="' + (inRange ? 'real ' : '') + (late ? 'late ' : '') + (steps[j].today ? 'today ' : '') + ('day' + steps[j].weekday) +
							'">';
						real += '<span>&nbsp;</span>';
						real += '</td>';
					}

					html += '<tr data-ng-class="{\'open\': widget.plainTreeData[' + rowIdx + ']._opened, \'closed\': !widget.plainTreeData[' + rowIdx +
						']._opened}" data-ng-show="canShowRow(widget.plainTreeData[' + rowIdx + '])">';
					html += planned;
					html += '</tr>';
					html += '<tr data-ng-class="{\'open\': widget.plainTreeData[' + rowIdx + ']._opened, \'closed\': !widget.plainTreeData[' + rowIdx +
						']._opened}" data-ng-show="canShowRow(widget.plainTreeData[' + rowIdx + '])">';
					html += real;
					html += '</tr>';

					element.find('.table-container table tbody').append($compile(html)($scope));

					$timeout(function() {
						buildGantt(rowIdx + 1);
					});
				} else {
					$rootScope.$broadcast("loader.hide");
					building = false;
				}
			}

			function updateGantt() {
				if (!building) {
					building = true;
					if ($scope.widget.plainTreeData.length > 0) {
						$rootScope.$broadcast("loader.show");
						$scope.ganttSteps = getGanttSteps();
						$timeout(function() {
							var rowIdx = 0;
							buildGantt(rowIdx);
						});
					} else {
						cleanGantt();
					}
				}
			}

			$scope.$watch('zoomLevel', function(newValue, oldValue) {
				if (newValue && newValue != oldValue) {
					updateGantt();
				}
			});
			$scope.$watch('widget.plainTreeData', function(value) {
				if ($scope.widget.fieldPlanStart && value) {
					updateGantt();
				}
			});

			function isInRange(row, step, type) {
				var inRange = false;
				var startDate, endDate;
				if (type == 'planned') {
					startDate = new Date(row[$scope.widget.fieldPlanStart]);
					endDate = new Date(row[$scope.widget.fieldPlanEnd]);
				} else {
					startDate = new Date(row[$scope.widget.fieldRealStart]);
					endDate = new Date(row[$scope.widget.fieldRealEnd]);
				}
				inRange = startDate >= step.begin && startDate <= step.end || endDate >= step.begin && endDate <= step.end || startDate <= step.begin &&
					endDate >= step.end;
				return inRange;
			}

			function isLate(row) {
				return row[$scope.widget.fieldLateStatus];
			}

			$scope.widget.changeGanttZoomLevel = function(level) {
				$scope.zoomLevel = level;
			};

			var _widget = $scope.widget;
			$scope.$on('$destroy', function() {
				delete _widget.changeGanttZoomLevel;
			});

			$scope.canShowRow = function(row) {
				if (!row) {
					return false;
				}
				if (Util.objectHasAnyValue($scope.widget.searchCriteria)) {
					return $scope.controller.search(row);
				} else {
					return $scope.controller.isOpened(row) || row._parent === null;
				}
			};
		}
	};
});

ZeedhiInterations.directive('zhSlideForm', function(templateManager) {
	return {
		restrict: 'A',
		link: function($scope, $element, iAttrs, controller) {
			var widget = $scope.widget;
			if (widget) {
				widget.isOpen = false;

				widget.hideForm = function() {
					if (!widget.isFilterTab) {
						if ($element.hasClass("in") || $element.hasClass("infull") || $element.hasClass("out")) {
							$element.removeClass();
							widget.editing = false;
							$element.addClass("zh-widget slideGridForm outfull");
							widget.isSlidingForm = false;
						}
						widget.editing = false;
						widget.isOpen = false;
					}
				};

				widget.hideIfOpen = function() {
					if (widget.isOpen) {
						widget.hideForm();
					}
				};

				widget.middleHideForm = function() {
					if (!widget.isFilterTab) {
						$element.removeClass();
						$element.addClass("zh-widget slideGridForm out zh-shadow");
						widget.isSlidingForm = false;
						widget.isOpen = true;
					}
				};

				widget.middleShowForm = function() {
					if (!widget.isFilterTab) {
						$element.removeClass();
						$element.addClass("zh-widget slideGridForm infull zh-shadow");
						widget.isSlidingForm = true;
						$scope.$apply();
						widget.isOpen = true;
					}
				};

				widget.showForm = function() {
					if (!widget.isFilterTab) {
						$element.removeClass();
						$element.addClass("zh-widget slideGridForm in zh-shadow");
						widget.isSlidingForm = true;
						Util.doAsync(function() {
							widget.isOpen = true;
						});
					}
				};

				$scope.$on('$destroy', function() {
					delete widget.hideForm;
					delete widget.hideIfOpen;
					delete widget.middleHideForm;
					delete widget.middleShowForm;
					delete widget.showForm;
				});

				$scope.$watch('widget.editing', function(value) {
					if (value === true) {
						widget.showForm();
					} else if (value === false && ($element.hasClass('out') || $element.hasClass('in'))) {
						widget.hideForm();
					}
				});

				Zepto($element[0]).swipeRight(function(e) {
					e.stopPropagation();
					if ($(this).hasClass("infull")) {
						widget.middleHideForm();
					} else {
						widget.editing = false;
						widget.hideForm();
					}
					templateManager.menuVisible(false);
					$scope.$apply();
				});

				Zepto($element[0]).swipeLeft(function() {
					widget.middleShowForm();
				});

				$element.on("click", function() {
					templateManager.menuVisible(false);
					$scope.$apply();
				});
			}
		}
	};
});

// @todo remove it
ZeedhiInterations.directive('zhSlideFilter', function($document, templateManager) {
	return {
		restrict: 'A',
		link: function($scope, $element, iAttrs, controller) {

			var widget = $scope.__getWidget();
			widget.filterTabOpen = false;

			widget.toggleFilterTab = function() {
				if (widget.filterTabOpen) {
					widget.hideFilterTab();
				} else {
					widget.showFilterTab();
				}
			};

			widget.showFilterTab = function() {
				if (!widget.isSlidingForm) {
					$element.removeClass();
					$element.addClass("zh-widget slideGridFormLeftToRight in zh-shadow");
					widget.isFilterTab = true;
					templateManager.menuVisible(false);
					widget.filterTabOpen = true;
				}
			};

			widget.hideFilterTab = function() {
				if (!widget.isSlidingForm) {
					if ($(this).hasClass("infull")) {
						$element.removeClass();
						$element.addClass("zh-widget slideGridFormLeftToRight out zh-shadow");
						widget.isFilterTab = true;
					} else {
						$element.removeClass();
						$element.addClass("zh-widget slideGridFormLeftToRight outfull");
						widget.isFilterTab = false;
					}
					widget.filterTabOpen = false;
				}
			};

			$scope.$on('$destroy', function() {
				delete widget.toggleFilterTab;
				delete widget.showFilterTab;
				delete widget.hideFilterTab;
			});

			Zepto($document[0]).swipeRight(function() {
				if (templateManager.container.widgets[0].filter) {
					widget.showFilterTab();
				}
			});

			Zepto($document[0]).swipeLeft(function() {
				widget.hideFilterTab();
				templateManager.menuVisible(false);
				$scope.$apply();
			});

			$scope.$watch("menuHardware", function(menuHardware) {
				if (menuHardware) {
					widget.showFilterTab();
				} else {
					widget.hideFilterTab();
				}
			});
		}
	};
});

ZeedhiInterations.directive('zhSlideMenu', function() {
	return {
		restrict: 'A',
		link: function($scope, iElm, iAttrs, controller) {
			// scrolling on iOS
			iElm.nonbounce();
			$scope.$watch('templateManager.project.menuVisible', function(value) {
				var element = $(iElm[0]);
				if (value === true) {
					element.removeClass("out");
					element.addClass("in zh-shadow");
				} else if (value === false && element.hasClass('in')) {
					element.removeClass("in");
					element.removeClass("zh-shadow");
					element.addClass("out");
				}
			});
		}
	};
});

ZeedhiInterations.directive('zhMenuControl', function(templateManager) {
	return {
		restrict: 'A',
		link: function($scope, $element, iAttrs, controller) {
			$element.on("click", function() {
				templateManager.menuVisible(!templateManager.project.menuVisible);
			});
		}
	};
});

function isChrome() {
	return (/Chrome/.test(navigator.userAgent) && /Google Inc/.test(navigator.vendor));
}

(function($, window, Zepto) {
	"use strict";

	var startY, startX;

	var defaults = {
		"$these": [],
		"touchstartInit": false,
		"touchmoveInit": false
	};

	$(".nonbounce").each(function() {
		defaults.$these.push($(this));
	});

	$.fn.nonbounce = function() {
		if (Util.isMobile()) {
			initTouchHandling();

			this.addClass('nonbounce');

			return this.each(function() {
				defaults.$these.push($(this));
			});
		}
	};

	$.nonbounce = function() {
		if (Util.isMobile()) {
			initTouchHandling();
		}
	};

	var scrollParent = function(evt) {
		var y = (evt.originalEvent.touches) ? evt.originalEvent.touches[0].screenY : evt.originalEvent.screenY;
		var nonbounce = $(evt.target).closest(".nonbounce");

		if (nonbounce) {
			nonbounce.scrollTop(nonbounce.scrollTop() + startY - y);
			startY = y;
		}

		evt.preventDefault();
	};

	var processXScroll = function(evt) {
		var x = (evt.originalEvent.touches) ? evt.originalEvent.touches[0].screenX : evt.originalEvent.screenX;
		var delta = startX - x;

		if (Math.abs(delta) > 0) {
			var nonbounce = $(evt.target).closest(".nonbounce");
			while (nonbounce.length > 0) {
				if (nonbounce[0].scrollWidth > nonbounce[0].clientWidth) {
					nonbounce.scrollLeft(nonbounce.scrollLeft() + delta);
					startX = x;
					break;
				} else {
					nonbounce = nonbounce.parent().closest(".nonbounce");
				}
			}

		}
	};

	var touchstart = function(evt) {
		evt = evt.originalEvent || evt;
		startY = (evt.touches) ? evt.touches[0].screenY : evt.screenY;
		startX = (evt.touches) ? evt.touches[0].screenX : evt.screenX;
	};

	var touchmove = function(evt) {
		if (!(evt.originalEvent.touches && evt.originalEvent.touches.length > 1)) {
			if ((evt.target.tagName && evt.target.tagName.toLowerCase() === 'input') &&
				(Zepto.os.phone || Zepto.os.tablet)) {
				scrollParent(evt);
			}

			// Prevents scrolling of all but the nonbounce elements
			if (!~$.inArray(true, $.map(defaults.$these, compareElem, evt.target))) {
				evt.preventDefault();
			}
		}

		processXScroll(evt);
	};

	var initTouchHandling = function() {
		if (!defaults.touchstartInit) {
			defaults.touchstartInit = true;
			$(window).on("touchstart", touchstart);
		}

		if (!defaults.touchmoveInit) {
			defaults.touchmoveInit = true;
			$(window).on("touchmove", touchmove);
		}
	};

	var compareElem = function($elem, i, target) {
		return !!$(target).closest($elem).length;
	};

})(jQuery, window, Zepto);

ZeedhiInterations.directive('zhUpdateScroller', ["$timeout", "templateManager", function($timeout, templateManager) {
	return {
		restrict: 'A',
		link: function($scope, element, iAttrs) {
			var delay = iAttrs.zhUpdateScroller;

			element.on('click', function() {
				$timeout(function() {
					for (var i in templateManager.scrollers) {
						if (templateManager.scrollers.hasOwnProperty(i)) {
							templateManager.scrollers[i].refresh();
						}
					}
				}, delay, false);
			});
		}
	};
}]);

ZeedhiInterations.directive('zhTabsVertical', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, iElement, iAttrs) {
			$scope.$watchCollection('templateManager.historyItems', function(item) {
				$timeout(function() {
					var tabsContainer = $(iElement[0]);
					if (tabsContainer.is(':visible')) {
						tabsHeight = 0;
						tabsContainer.children('li').each(function() {
							tabsHeight += $(this).width();
						});
						tabsWidth = tabsContainer.children().height();
						tabsContainer.css({
							width: tabsHeight,
							left: -(tabsHeight / 2) + (tabsWidth / 2),
							marginTop: -tabsWidth / 2,
							top: tabsHeight / 2
						});
						$scope.resizeWindowElements();
					}
				}, 0, false);
			});
		}
	};
}]);

ZeedhiInterations.directive('zhTabsVerticalItem', function() {
	return {
		restrict: 'A',
		link: function($scope, iElement, iAttrs) {
			iElement.on('click', function() {
				$(this).addClass('active');
				$(this).siblings().removeClass('active');
				$scope.openWindow($scope.item);
				$scope.$apply();
			});
		}
	};
});

ZeedhiInterations.directive('zhMessage', function() {
	return {
		restrict: 'A',
		link: function($scope, iElement, iAttrs) {
			var content = $(".zh-content");
			var timer = null;

			var hideMessage = function() {
				iElement.slideUp(200);
				iElement.children('.messagesEntry').html("");
				iElement.data('resized', false);
				content.removeAttr('style');
			};

			$scope.$watchCollection('messagesToShow', function(messages, oldMessages) {
				if (messages.length > 0) {
					angular.forEach(messages, function(message, key) {
						if (iElement.not(":visible")) {
							iElement.slideDown(200).addClass(message.type);
						}
						iElement.children('.messagesEntry').html(message.message);

						if (!iElement.data('resized')) {
							iElement.data('resized', true);

							var contentHeight = content.height();
							content.animate({
								height: contentHeight - 50 + "px"
							}, 200);
						}
						$scope.messagesToShow.splice(key);
					});
					timer = setTimeout(hideMessage, 6000);
				}
			});

			iElement.children(".messagesClose").on("click", function() {
				hideMessage();
				clearTimeout(timer);
			});
		}
	};
});

ZeedhiInterations.directive('zhMenuDropDowm', function($timeout, templateManager) {
	return {
		restrict: 'A',
		link: function($scope, iElm, iAttrs, controller) {
			$timeout(function() {
				var processMenu = function(menuObj, clickedNode) {
					var nodeInMenu = menuObj.find(clickedNode);
					if (!nodeInMenu.hasClass('active')) {
						nodeInMenu.removeClass('inactive').addClass('active');
						nodeInMenu.siblings().removeClass('active').addClass('inactive');
						nodeInMenu.children('.group').css('display', 'block');
					} else {
						var childrens = nodeInMenu.find('.active');
						if (childrens.length) {
							angular.forEach(childrens, function(children) {
								Zepto(children).removeClass('active').addClass('inactive');
								Zepto(children).children('.group').css('display', 'none');
							});
						}
						if (nodeInMenu.children('.group').length) {
							nodeInMenu.removeClass('active').addClass('inactive');
							nodeInMenu.children('.group').css('display', 'none');
						}
					}
					return menuObj;
				};

				var expandAllMenu = function(menuObj) {
					menuObj.find('.group').css('display', 'block');
					menuObj.find('.group').parent('li').removeClass('inactive').addClass('active');
					return menuObj;
				};

				var colapseAllMenu = function(menuObj) {
					menuObj.find('.group').css('display', 'none');
					menuObj.find('.active').removeClass('active').addClass('inactive');
					return menuObj;
				};

				var content = iElm.find('li.zh-menu-dropdowm-item');
				content.on('click', function(e) {
					var parent = iElm;
					var menuObj = parent.children('nav').detach();
					menuObj = processMenu(menuObj, this);
					parent.append(menuObj);
					e.stopPropagation();
				});

				$scope.$watch('menuFilter', function(value) {
					var parent = iElm;
					var menuObj = parent.children('nav').detach();
					if (value) {
						menuObj = expandAllMenu(menuObj, this);
						parent.append(menuObj);
					} else {
						menuObj = colapseAllMenu(menuObj, this);
						parent.append(menuObj);
					}
				});

				$scope.$watch('templateManager.container', function() {
					templateManager.menuVisible(false);
					var parent = iElm;
					var menuObj = parent.children('nav').detach();
					menuObj = colapseAllMenu(menuObj);
					parent.append(menuObj);
				});
			}, 200, false);

		}
	};
});

// ZeedhiInterations.directive('zhBody', function($rootScope) {
// 	return {
// 		restrict: 'A',
// 		link: function(scope, iElm, iAttrs) {
// 			var reloadSize = function() {
// 				var windowHeight = $(window).height();
// 				var headerHeight = $("header").height();
// 				var footerHeight = $("footer").height();
// 				var contentHeight = windowHeight - headerHeight - footerHeight;

// 				$('.tabsHeight').css('width', contentHeight);

// 				$rootScope.$broadcast('window.reload');
// 			};
// 			$(window).resize(reloadSize);
// 			window.reloadSize = reloadSize;
// 		}
// 	};
// });

ZeedhiInterations.directive('zhGridHeader', function(templateManager) {
	return {
		restrict: 'A',
		link: function($scope, iElm, iAttrs, controller) {
			iElm.on('click', function() {
				$scope.widget.orderByValue = $scope.column.name;
				$scope.column.orderReverse = !$scope.column.orderReverse;
				$scope.widget.orderReverse = $scope.column.orderReverse;
				$scope.$apply();
			});
		}
	};
});
ZeedhiInterations.directive('zhExpandWidget', function(templateManager) {
	return {
		restrict: 'A',
		link: function($scope, iElm, iAttrs, controller) {
			iElm.on('click', function(e) {
				$scope.widget.expand = !$scope.widget.expand;
				if ($scope.widget.expand) {
					$scope.widget.oldHeight = $scope.widget.height;
					$scope.widget.height = "100%";
				} else {
					$scope.widget.height = $scope.widget.oldHeight;
				}
				$scope.$apply();
				e.stopPropagation();
			});
		}
	};
});

ZeedhiInterations.directive('zhStoryContainerHeader', ["ZhFeatures", function(ZhFeatures) {
	return {
		restrict: 'C',
		link: function($scope, iElm, iAttrs, controller) {
			ZhFeatures.registerFeatureUsage({ directive: "zhStoryContainerHeader" });
			iElm.on('click', function() {
				var storyContainerBody = iElm.next();
				storyContainerBody.toggleClass('hide');
			});
		}
	};
}]);

ZeedhiInterations.directive('zhStopPropagationClick', function() {
	return {
		restrict: 'A',
		priority: 600,
		link: function(scope, $elem, attr, ctrl) {
			$elem.on('touchstart touchmove click', function(e) {
				var parents = $(e.target).parents();
				if ($(e.target).is(':not(span):not(:input)') && !parents.is('.zh-dont-stop-propagation')) {
					e.preventDefault();
					return false;
				}
			});
		}
	};
});

ZeedhiInterations.directive("zhKanbanColumnContainer", function() {
	return {
		restrict: "C",
		link: function($scope, element, iAttrs, controller) {
			$(window).on('resize.kanban', resizeItems);

			function resizeItems() {
				var columnMargin = 12;
				var columnWidth = Math.floor((element.parent().width() + columnMargin) / $scope.widget.columns.length);
				if (columnWidth < 250) {
					columnWidth = 250;
				}
				element.css('width', $scope.widget.columns.length * columnWidth);
				element.children('.kanban-column').css('width', columnWidth - columnMargin).css('margin-right', columnMargin).animate({
					'opacity': 1
				}, 500);
				element.find('.kanban-item-container').nonbounce();
			}

			$scope.resizeItems = resizeItems;

			$scope.$watch('widget.columns.length', function() {
				resizeItems();
			});

			$scope.$on('$destroy', function() {
				$(window).off('resize.kanban');
			});
		}
	};
});

ZeedhiDirectives.directive('zhFileUpload', function(templateManager, ZHPromise) {
	return {
		link: function(scope, element, attrs) {
			var field = scope.field;
			element.shouldNotPreventDefaultOnClick = true;
			var widget = scope.widget;
			var container = element.parent();

			field.fileList = [];
			if (field.isMultiple) {
				element.attr("multiple", "true");
			}

			element.parent().find("ul").nonbounce();

			var button = element.siblings('button');
			var input = element;

			button.on('click.file-upload touchstart.file-upload', function(e) {
				input.trigger(e.type);
			});

			var processFile = function(file) {
				var deferred = ZHPromise.defer();
				var fileReader = new FileReader();
				fileReader.onload = function(e) {
					var fileToChange = widget.currentRow[field.name].filter(function(f){
						return f.name === file.name;
					})[0];
					if(!Util.isEmptyOrBlank(fileToChange)){
						fileToChange = $.extend(false, {b64File: e.target.result}, file);
					}
					else{
						widget.currentRow[field.name].push($.extend(false, {
							b64File: e.target.result
						}, file));
					}
					field.fileList = widget.currentRow[field.name];
					deferred.resolve();
				};
				fileReader.readAsDataURL(file);
				return deferred;
			};

			element.on('change', function() {
				if (this.files.length > 0) {
					if (!widget.currentRow) {
						widget.currentRow = {};
					}
					if (!field.isMultiple || !Array.isArray(widget.currentRow[field.name])) {
						widget.currentRow[field.name] = [];
					}
					widget.currentRow.changed = true;
					var promises = [];
					for (var i in this.files) {
						if (this.files.hasOwnProperty(i) && this.files[i] instanceof File) {
							promises.push(processFile(this.files[i]));
						}
					}
					ZHPromise.all(promises).then(function() {
						if(field.afterSelectFile){
							field.afterSelectFile();
						}
						field.validate(field.value());
						templateManager.updateTemplate();
					});
				}
				this.value = "";
			});
		}
	};
});

ZeedhiDirectives.directive('zhRippleEffect', function() {
	return {
		link: function(scope, element, attrs) {
			element.addClass('zh-ripple-effect');

			element.click(function(e) {
				var ink;

				//incase of quick double clicks stop the previous animation
				$('.ink').removeClass('animate');

				//create .ink element if it doesn't exist
				if (element.find('.ink').length === 0) {
					element.prepend('<span class="ink '+attrs.zhRippleEffect+'" contenteditable="false"></span>');
				}
				ink = element.find('.ink');

				//set size of .ink
				//use element width or height whichever is larger for the diameter to make a circle which can cover the entire element.
				var d = Math.max(element.outerWidth(), element.outerHeight());
				ink.css({height: d, width: d});

				//get click coordinates
				var x = e.pageX - element.offset().left - ink.width()/2;
				var y = e.pageY - element.offset().top - ink.height()/2;

				//set the position and add class .animate
				ink.css({top: y+'px', left: x+'px'}).addClass("animate");
			});
		}
	};
});

ZeedhiDirectives.directive('zhFocusField', function($timeout) {
	return {
		link: function($scope, element, attrs, controller) {
			if (attrs.zhFocusField && Util.isDesktop()) {
				$timeout(function() { element.focus(); }, 500, false); // wait for the screen to finish render...
			}
		}
	};
});
